#include "StdAfx.h"
#include "simplepgp.h"

#pragma comment(lib,"pgp_sdk.lib")

#ifdef _WITH_PGPUI_
#include "pgpUserInterface.h"
#pragma comment(lib,"pgpsdkui.lib")
#endif

// StrPrintf  StrPrintfV -
//    std::string
//   TepMuHyc-
// http://www.rsdn.ru/forum/?mid=54548
string StrPrintfV( char* pszFormat, va_list marker )
{
    static vector<char> data;
    data.reserve( 256 );

    while ( _vsnprintf( &data[ 0 ], data.capacity(), pszFormat, marker ) == -1 )
        data.reserve( data.capacity() + 256 );

    return string( data.begin(), data.end() );
}

string StrPrintf( char* pszFormat, ... )
{
    va_list marker;
    va_start( marker, pszFormat );
    string str = StrPrintfV( pszFormat, marker );
    va_end( marker );
    return str;
}

// 
CSimplePGP::CSimplePGP( void )
{
    //  
    m_bIsInit = FALSE;
    m_context = kInvalidPGPContextRef;
    m_sVersion = "CSimplePGP v3.0";
    m_sComment = "no comment";
    m_optsEncode = NULL;
    m_optsPassphrase = NULL;
    m_optsSigning = NULL;
    m_sWhere = "";
    lstrcpyn( m_sWhat, _T( "     " ), sizeof( m_sWhat ) );
    m_bASCIIoutput = TRUE;
    m_bASCIIinput = FALSE;
    m_algoID = kPGPCipherAlgorithm_IDEA;
    m_hashID = kPGPHashAlgorithm_MD5;
    m_bClearSign = FALSE;
    m_bDetachedSig = TRUE;
}

// 
CSimplePGP::~CSimplePGP( void )
{
    if ( m_bIsInit )
    {
        if ( m_optsEncode != NULL )
        {
            //    
            PGPFreeOptionList( m_optsEncode );
            m_optsEncode = NULL;
        }

        if ( m_optsPassphrase != NULL )
        {
            //    
            PGPFreeOptionList( m_optsPassphrase );
            m_optsEncode = NULL;
        }

        if ( m_optsSigning != NULL )
        {
            //    
            PGPFreeOptionList( m_optsSigning );
            m_optsSigning = NULL;
        }

        //  
        if ( PGPContextRefIsValid( m_context ) )
            PGPFreeContext( m_context );

        // PGP library shutdown
#ifdef _WITH_PGPUI_

        PGPsdkUILibCleanup();

#endif

        PGPsdkCleanup();
    }
}

// 
BOOL CSimplePGP::Init()
{
    //     
    if ( m_bIsInit )
        return m_bIsInit;

    //   PGP
    PGPError err = kPGPError_NoErr;

    //  PGP SDK
    err = PGPsdkInit();

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPsdkInit()";
        goto Exit;
    }

    //  pgp-
    err = PGPNewContext( kPGPsdkAPIVersion, &m_context );

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPNewContext()";
        goto Exit;
    }

    //   ,     
    err = PGPBuildOptionList( m_context,
                              &m_optsEncode,
                              //     ?
                              PGPOArmorOutput( m_context, m_bASCIIoutput ),
                              //   -  ?
                              PGPODataIsASCII( m_context, m_bASCIIinput ),
                              // 
                              PGPOCommentString( m_context, m_sComment.c_str() ),
                              // 
                              PGPOVersionString( m_context, m_sVersion.c_str() ),
                              //  
                              PGPOCipherAlgorithm( m_context, m_algoID ),
                              //   
                              PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPBuildOptionList() - m_optsEncode";
        goto Exit;
    }

    //   ,     
    err = PGPBuildOptionList( m_context,
                              &m_optsPassphrase,
                              //  
                              PGPOPassphrase( m_context, "" ),
                              //   
                              PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPBuildOptionList() - m_optsPassphrase";
        goto Exit;
    }

    //   ,       
    err = PGPBuildOptionList( m_context,
                              &m_optsSigning,
                              // 
                              PGPOCommentString( m_context, m_sComment.c_str() ),
                              // 
                              PGPOVersionString( m_context, m_sVersion.c_str() ),
                              //  
                              PGPOCipherAlgorithm( m_context, m_algoID ),
                              //  
                              PGPOHashAlgorithm( m_context, m_hashID ),
                              // "" ,  ,    
                              PGPOClearSign( m_context, m_bClearSign ),
                              //   
                              PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPBuildOptionList() - m_optsSigning";
        goto Exit;
    }

    //         
    if ( !m_bClearSign )
    {
        // PGPODataIsASCII, PGPOArmorOutput, PGPODetachedSig  
        //      PGPOClearSign
        err = PGPAppendOptionList( m_optsSigning,
                                   //   -  ?
                                   PGPODataIsASCII( m_context, m_bASCIIinput ),
                                   //     ?
                                   PGPOArmorOutput( m_context, m_bASCIIoutput ),
                                   //   
                                   PGPOLastOption( m_context ) );

        if ( IsPGPError( err ) )
        {
            //   
            m_sWhere = "PGPAppendOptionList() - m_optsSigning";
            goto Exit;
        }

        if ( m_bDetachedSig )
        {
            err = PGPAppendOptionList( m_optsSigning,
                                       //    ?
                                       PGPODetachedSig( m_context, PGPOLastOption( m_context ) ),
                                       //   
                                       PGPOLastOption( m_context ) );
            if ( IsPGPError( err ) )
            {
                //   
                m_sWhere = "PGPBuildOptionList() - m_optsSigning - PGPODetachedSig()";
                goto Exit;
            }
        }
    }

#ifdef _WITH_PGPUI_
    err = PGPsdkUILibInit();

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPsdkUILibInit()";
        goto Exit;
    }

#endif

    //   ,   
    m_bIsInit = TRUE;

Exit:

    if ( IsPGPError( err ) )
    {
        //      ,
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
        return FALSE;
    }

    return m_bIsInit;
}

//         
BOOL CSimplePGP::ReBuildEncodeOptionList()
{
    if ( !m_bIsInit )
        Init();

    PGPError err = kPGPError_NoErr;

    if ( m_optsEncode != NULL )
    {
        //    
        PGPFreeOptionList( m_optsEncode );
        m_optsEncode = NULL;
    }

    //   ,     
    err = PGPBuildOptionList( m_context,
                              &m_optsEncode,
                              //     ?
                              PGPOArmorOutput( m_context, m_bASCIIoutput ),
                              //   -  ?
                              PGPODataIsASCII( m_context, m_bASCIIinput ),
                              // 
                              PGPOCommentString( m_context, m_sComment.c_str() ),
                              // 
                              PGPOVersionString( m_context, m_sVersion.c_str() ),
                              //  
                              PGPOCipherAlgorithm( m_context, m_algoID ),
                              //   
                              PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "ReBuildEncodeOptionList()";
        return FALSE;
    }

    return TRUE;
}

//          
BOOL CSimplePGP::ReBuildSigningOptionList()
{
    if ( !m_bIsInit )
        Init();

    PGPError err = kPGPError_NoErr;

    if ( m_optsSigning != NULL )
    {
        //    
        PGPFreeOptionList( m_optsSigning );
        m_optsSigning = NULL;
    }

    //   ,       
    err = PGPBuildOptionList( m_context,
                              &m_optsSigning,
                              // 
                              PGPOCommentString( m_context, m_sComment.c_str() ),
                              // 
                              PGPOVersionString( m_context, m_sVersion.c_str() ),
                              //  
                              PGPOCipherAlgorithm( m_context, m_algoID ),
                              //  
                              PGPOHashAlgorithm( m_context, m_hashID ),
                              // "" ,  ,    
                              PGPOClearSign( m_context, m_bClearSign ),
                              //   
                              PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPBuildOptionList() - m_optsSigning";
        return FALSE;
    }

    //         
    if ( !m_bClearSign )
    {
        // PGPODataIsASCII, PGPOArmorOutput, PGPODetachedSig  
        //      PGPOClearSign
        err = PGPAppendOptionList( m_optsSigning,
                                   //   -  ?
                                   PGPODataIsASCII( m_context, m_bASCIIinput ),
                                   //     ?
                                   PGPOArmorOutput( m_context, m_bASCIIoutput ),
                                   //   
                                   PGPOLastOption( m_context ) );

        if ( IsPGPError( err ) )
        {
            //   
            m_sWhere = "PGPAppendOptionList() - m_optsSigning";
            return FALSE;
        }

        if ( m_bDetachedSig )
        {
            err = PGPAppendOptionList( m_optsSigning,
                                       //    ?
                                       PGPODetachedSig( m_context, PGPOLastOption( m_context ) ),
                                       //   
                                       PGPOLastOption( m_context ) );
            if ( IsPGPError( err ) )
            {
                //   
                m_sWhere = "PGPBuildOptionList() - m_optsSigning - PGPODetachedSig()";
                return FALSE;
            }
        }
    }

    return TRUE;
}

//   -     ,    
BOOL CSimplePGP::SetClearSigning ( BOOL bClearSign )
{
    m_bClearSign = bClearSign;
    return ReBuildSigningOptionList();
}

//   -     ,    
BOOL CSimplePGP::SetDetachedSig ( BOOL bDetachedSig )
{
    m_bDetachedSig = bDetachedSig;
    return ReBuildSigningOptionList();
}

//    
BOOL CSimplePGP::SetVersion ( LPCTSTR sVersion )
{
    m_sVersion = sVersion;
    return ( ReBuildEncodeOptionList() && ReBuildSigningOptionList() );
}

//   - 
BOOL CSimplePGP::SetComment ( LPCTSTR sComment )
{
    //   
    m_sComment = sComment;
    //   
    return ( ReBuildEncodeOptionList() && ReBuildSigningOptionList() );
}

//   
BOOL CSimplePGP::SetASCIIoutput ( BOOL bASCIIout )
{
    m_bASCIIoutput = bASCIIout;
    return ( ReBuildEncodeOptionList() && ReBuildSigningOptionList() );
}

//   
BOOL CSimplePGP::SetASCIIinput ( BOOL bASCIIinp )
{
    m_bASCIIinput = bASCIIinp;
    return ( ReBuildEncodeOptionList() && ReBuildSigningOptionList() );
}

//    
BOOL CSimplePGP::SetCipherAlgorithm( PGPCipherAlgorithm algoID )
{
    m_algoID = algoID;
    return ( ReBuildEncodeOptionList() && ReBuildSigningOptionList() );
}

//  
BOOL CSimplePGP::SetHashAlgorithm( PGPHashAlgorithm hashID )
{
    m_hashID = hashID;
    return ( ReBuildEncodeOptionList() && ReBuildSigningOptionList() );
}

//    
BOOL CSimplePGP::ImportKeySetFromFile( LPCTSTR keyFileName,                //    
                                       PGPKeySetRef& keySet )              //    
{
    PGPError err = kPGPError_NoErr;

    //     keyset,  
    if ( PGPKeySetRefIsValid( keySet ) )
        PGPFreeKeySet( keySet );

    keySet = kInvalidPGPKeySetRef;

    //    
    PGPFileSpecRef keyFileRef = kInvalidPGPFileSpecRef;

    BOOL ret = TRUE;

    //  
    err = PGPNewFileSpecFromFullPath( m_context, keyFileName, &keyFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath( %s )", keyFileName );
        goto Exit;
    }

    //      
    err = PGPImportKeySet( m_context,
                           &keySet,
                           PGPOInputFile( m_context, keyFileRef ),
                           PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPImportKeySet( %s )", keyFileName );
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPFileSpecRefIsValid( keyFileRef ) )
        PGPFreeFileSpec( keyFileRef );

    return ret;
}

//    
BOOL CSimplePGP::ImportKeySetFromResorce( LPCTSTR ResourceName,               //    
                                       LPCTSTR ResourceType,               //  
                                       PGPKeySetRef& keySet )              //  ,  
{
    PGPError err = kPGPError_NoErr;

    //     keyset,  

    if ( PGPKeySetRefIsValid( keySet ) )
        PGPFreeKeySet( keySet );

    keySet = kInvalidPGPKeySetRef;

    //     
    HRSRC pgpKey = NULL;

    HGLOBAL pRes = NULL;

    LPVOID pKeyFileInRes = NULL;

    /*       */
    // ...
    pgpKey = FindResource( NULL, ( LPCTSTR ) ResourceName, ( LPCTSTR ) ResourceType );

    if ( !pgpKey )
    {
        m_sWhere = StrPrintf( "   '%s'  '%s'    ",
                              ResourceName, ResourceType );
        lstrcpyn( m_sWhat, m_sWhere.c_str(), sizeof( m_sWhat ) );
        m_sWhere = " ImportKeySetFromResorce()";
        return FALSE;
    }

    // ...
    pRes = LoadResource( NULL, pgpKey );

    if ( !pRes )
    {
        m_sWhere = StrPrintf( "    '%s'  '%s'    ",
                              ResourceName, ResourceType );
        lstrcpyn( m_sWhat, m_sWhere.c_str(), sizeof( m_sWhat ) );
        m_sWhere = " ImportKeySetFromResorce()";
        return FALSE;
    }

    //  ...
    pKeyFileInRes = LockResource( pRes );

    if ( !pKeyFileInRes )
    {
        m_sWhere = StrPrintf( "      '%s'  '%s'    ",
                              ResourceName, ResourceType );
        lstrcpyn( m_sWhat, m_sWhere.c_str(), sizeof( m_sWhat ) );
        m_sWhere = " ImportKeySetFromResorce()";
        FreeResource( pRes );
        return FALSE;
    }

    //    
    err = PGPImportKeySet( m_context,
                           &keySet,
                           //   
                           PGPOInputBuffer( m_context,
                                            pKeyFileInRes,
                                            SizeofResource( NULL, pgpKey ) ),
                           //    
                           PGPOLastOption( m_context ) );

    //  
    UnlockResource( pRes );

    FreeResource( pRes );

    //   
    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPImportKeySet( '%s', '%s' )   ",
                              ResourceName, ResourceType );
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
        return FALSE;
    }

    return TRUE;
}

// 
//    ,     
BOOL CSimplePGP::EncodeFile2File( LPCTSTR inFileName,                 //   
                                  LPCTSTR outFileName,                //   
                                  LPCTSTR keyFileName )               //     
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //   (-)  
    if ( !ImportKeySetFromFile( keyFileName, pubKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputFile( m_context, inFileRef ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //      
                     PGPOEncryptToKeySet( m_context, pubKeySet ),
                     //    
                     //   
                     m_optsEncode,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//      
BOOL CSimplePGP::IsOutBuffNotNull( LPBYTE& OutData )
{
    if ( OutData )
    {
        lstrcpyn( m_sWhat,
                  "   ,\n"
                  "OutData    NULL.\n"
                  "  -         \n"
                  "  delete[].",
                  sizeof( m_sWhat ) );
        return TRUE;
    }

    return FALSE;
}

//    ,    .
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::EncodeFile2Buff( LPCTSTR inFileName,                  //   
                                  LPBYTE& OutData,                     //     
                                  DWORD& BuffSize,                     //  
                                  LPCTSTR keyFileName )                //     
{

    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "EncodeFile2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //   (-)  

    if ( !ImportKeySetFromFile( keyFileName, pubKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPEncode( m_context,
                         //  
                         PGPOInputFile( m_context, inFileRef ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //      
                         PGPOEncryptToKeySet( m_context, pubKeySet ),
                         //    
                         //   
                         m_optsEncode,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    return ret;
}

//    ,    
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::EncodeFile2Buff( LPCTSTR inFileName,                //   
                                  LPBYTE& OutData,                   //     
                                  DWORD& BuffSize,                   //  
                                  LPCTSTR resourceName,              //   c  
                                  LPCTSTR resourceType )             //  
{

    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "EncodeFile2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //    

    if ( !ImportKeySetFromResorce( resourceName, resourceType, pubKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPEncode( m_context,
                         //  
                         PGPOInputFile( m_context, inFileRef ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //      
                         PGPOEncryptToKeySet( m_context, pubKeySet ),
                         //    
                         //   
                         m_optsEncode,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    return ret;
}

// 
//    ,     
BOOL CSimplePGP::EncodeBuff2File( const VOID* inData,                 //     
                                  DWORD dwDataSize,                   //  
                                  LPCTSTR outFileName,                //   
                                  LPCTSTR keyFileName )               //     
{
    if ( !m_bIsInit )
        Init();

    //  
    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromFile( keyFileName, pubKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputBuffer( m_context,
                                      ( void* ) inData,
                                      ( PGPSize ) dwDataSize ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //      
                     PGPOEncryptToKeySet( m_context, pubKeySet ),
                     //    
                     //   
                     m_optsEncode,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//    ,    
BOOL CSimplePGP::EncodeFile2File( LPCTSTR inFileName,                 //   
                                  LPCTSTR outFileName,                //   
                                  LPCTSTR resourceName,               //   c  
                                  LPCTSTR resourceType )              //  
{

    if ( !m_bIsInit )
        Init();

    //  
    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, pubKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputFile( m_context, inFileRef ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //      
                     PGPOEncryptToKeySet( m_context, pubKeySet ),
                     //    
                     //   
                     m_optsEncode,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    //  

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//    ,    
BOOL CSimplePGP::EncodeBuff2File( const VOID* inData,                 //     
                                  DWORD dwDataSize,                   //  
                                  LPCTSTR outFileName,             	  //   
                                  LPCTSTR resourceName,               //   c  
                                  LPCTSTR resourceType )              //  
{
    if ( !m_bIsInit )
        Init();

    //  
    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, pubKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputBuffer( m_context,
                                      ( void* ) inData,
                                      ( PGPSize ) dwDataSize ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //      
                     PGPOEncryptToKeySet( m_context, pubKeySet ),
                     //    
                     //   
                     m_optsEncode,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//    ,    
BOOL CSimplePGP::EncodeBuff2Buff( const VOID* inData,                //     
                                  DWORD dwDataSize,                  //  
                                  LPBYTE& OutData,                   //     
                                  DWORD& BuffSize,                   //  
                                  LPCTSTR keyFileName )              //     
{
    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "EncodeBuff2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //   (-)  

    if ( !ImportKeySetFromFile( keyFileName, pubKeySet ) )
        return FALSE;

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPEncode( m_context,
                         //  
                         PGPOInputBuffer( m_context,
                                          ( void* ) inData,
                                          ( PGPSize ) dwDataSize ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //      
                         PGPOEncryptToKeySet( m_context, pubKeySet ),
                         //    
                         //   
                         m_optsEncode,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    return ret;
}

//    ,    
BOOL CSimplePGP::EncodeBuff2Buff( const VOID* inData,                //     
                                  DWORD dwDataSize,                  //  
                                  LPBYTE& OutData,                   //     
                                  DWORD& BuffSize,                   //  
                                  LPCTSTR resourceName,              //   c  
                                  LPCTSTR resourceType )             //  
{
    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "EncodeBuff2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPKeySetRef pubKeySet = kInvalidPGPKeySetRef;

    //    

    if ( !ImportKeySetFromResorce( resourceName, resourceType, pubKeySet ) )
        return FALSE;

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPEncode( m_context,
                         //  
                         PGPOInputBuffer( m_context,
                                          ( void* ) inData,
                                          ( PGPSize ) dwDataSize ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //      
                         PGPOEncryptToKeySet( m_context, pubKeySet ),
                         //    
                         //   
                         m_optsEncode,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( pubKeySet ) )
        PGPFreeKeySet( pubKeySet );

    return ret;
}

// 

//     
BOOL CSimplePGP::SetPassphrase ( LPCTSTR sPassphrase )
{
    if ( !m_bIsInit )
        Init();

    PGPError err = kPGPError_NoErr;

    if ( m_optsPassphrase != NULL )
    {
        //    
        PGPFreeOptionList( m_optsPassphrase );
        m_optsPassphrase = NULL;
    }

    //   ,     
    err = PGPBuildOptionList( m_context,
                              &m_optsPassphrase,
                              //  
                              PGPOPassphrase( m_context, sPassphrase ),
                              //   
                              PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "SetPassphrase()";
        return FALSE;
    }

    return TRUE;
}

//    ,     
BOOL CSimplePGP::DecodeFile2File( LPCTSTR inFileName,               //   
                                  LPCTSTR outFileName,              //   
                                  LPCTSTR keyFileName )             //     
{

    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //   (-)  
    if ( !ImportKeySetFromFile( keyFileName, secKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPDecode( m_context,
                     //  
                     PGPOInputFile( m_context, inFileRef ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //   
                     PGPOKeySetRef( m_context, secKeySet ),
                     //    
                     //    
                     m_optsPassphrase,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//    ,    
BOOL CSimplePGP::DecodeFile2File( LPCTSTR inFileName,                 //   
                                  LPCTSTR outFileName,                //   
                                  LPCTSTR resourceName,               //   c  
                                  LPCTSTR resourceType )              //  
{

    if ( !m_bIsInit )
        Init();

    //  
    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPDecode( m_context,
                     //  
                     PGPOInputFile( m_context, inFileRef ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //   
                     PGPOKeySetRef( m_context, secKeySet ),
                     //    
                     //    
                     m_optsPassphrase,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
    }

Exit:

    //  

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//    ,     
BOOL CSimplePGP::DecodeBuff2File( const VOID* inData,                 //     
                                  DWORD dwDataSize,                   //  
                                  LPCTSTR outFileName,                //   
                                  LPCTSTR keyFileName )               //     
{
    if ( !m_bIsInit )
        Init();

    //  
    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromFile( keyFileName, secKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPDecode( m_context,
                     //  
                     PGPOInputBuffer( m_context,
                                      ( void* ) inData,
                                      ( PGPSize ) dwDataSize ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //   
                     PGPOKeySetRef( m_context, secKeySet ),
                     //    
                     //    
                     m_optsPassphrase,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//    ,    
BOOL CSimplePGP::DecodeBuff2File( const VOID* inData,                 //     
                                  DWORD dwDataSize,                   //  
                                  LPCTSTR outFileName,             	  //   
                                  LPCTSTR resourceName,               //   c  
                                  LPCTSTR resourceType )              //  
{
    if ( !m_bIsInit )
        Init();

    //  
    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPDecode( m_context,
                     //  
                     PGPOInputBuffer( m_context,
                                      ( void* ) inData,
                                      ( PGPSize ) dwDataSize ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //   
                     PGPOKeySetRef( m_context, secKeySet ),
                     //    
                     //    
                     m_optsPassphrase,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//    ,    .
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::DecodeFile2Buff( LPCTSTR inFileName,                  //   
                                  LPBYTE& OutData,                     //     
                                  DWORD& BuffSize,                     //  
                                  LPCTSTR keyFileName )                //     
{

    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "DecodeFile2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //   (-)  

    if ( !ImportKeySetFromFile( keyFileName, secKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    //  ,    PGPDecode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPDecode( m_context,
                         //  
                         PGPOInputFile( m_context, inFileRef ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //   
                         PGPOKeySetRef( m_context, secKeySet ),
                         //    
                         //    
                         m_optsPassphrase,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    return ret;
}

//    ,    
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::DecodeFile2Buff( LPCTSTR inFileName,                //   
                                  LPBYTE& OutData,                   //     
                                  DWORD& BuffSize,                   //  
                                  LPCTSTR resourceName,              //   c  
                                  LPCTSTR resourceType )             //  
{

    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "DecodeFile2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //    

    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeySet ) )
        return FALSE;

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    //  ,    PGPDecode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPDecode( m_context,
                         //  
                         PGPOInputFile( m_context, inFileRef ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //   
                         PGPOKeySetRef( m_context, secKeySet ),
                         //    
                         //    
                         m_optsPassphrase,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    return ret;
}

//    ,    
BOOL CSimplePGP::DecodeBuff2Buff( const VOID* inData,                //     
                                  DWORD dwDataSize,                  //  
                                  LPBYTE& OutData,                   //     
                                  DWORD& BuffSize,                   //  
                                  LPCTSTR keyFileName )              //     
{
    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "DecodeBuff2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //   (-)  

    if ( !ImportKeySetFromFile( keyFileName, secKeySet ) )
        return FALSE;

    //   
    //  ,    PGPDecode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPDecode( m_context,
                         //  
                         PGPOInputBuffer( m_context,
                                          ( void* ) inData,
                                          ( PGPSize ) dwDataSize ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //   
                         PGPOKeySetRef( m_context, secKeySet ),
                         //    
                         //    
                         m_optsPassphrase,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    return ret;
}

//    ,    
BOOL CSimplePGP::DecodeBuff2Buff( const VOID* inData,                //     
                                  DWORD dwDataSize,                  //  
                                  LPBYTE& OutData,                   //     
                                  DWORD& BuffSize,                   //  
                                  LPCTSTR resourceName,              //   c  
                                  LPCTSTR resourceType )             //  
{
    if ( !m_bIsInit )
        Init();

    if ( IsOutBuffNotNull( OutData ) )
    {
        m_sWhere = "DecodeBuff2Buff()";
        return FALSE;
    }

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPKeySetRef secKeySet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeySet ) )
        return FALSE;

    //   
    //  ,    PGPDecode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        err = PGPDecode( m_context,
                         //  
                         PGPOInputBuffer( m_context,
                                          ( void* ) inData,
                                          ( PGPSize ) dwDataSize ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //   
                         PGPOKeySetRef( m_context, secKeySet ),
                         //    
                         //    
                         m_optsPassphrase,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPDecode()";
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    // 
    if ( PGPKeySetRefIsValid( secKeySet ) )
        PGPFreeKeySet( secKeySet );

    return ret;
}

#ifdef _WITH_PGPUI_ 
//       PGP SDK UI
BOOL CSimplePGP::AskPassphrase( HWND hwndParent )
{
    char * passphrase;
    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;


    err = PGPPassphraseDialog( m_context,
                               PGPOUIOutputPassphrase( m_context, &passphrase ),
                               PGPOUIWindowTitle( m_context, " " ),
                               PGPOUIDialogPrompt( m_context, "  :" ),
                               PGPOUIParentWindowHandle( m_context, hwndParent ),
                               PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        //   
        m_sWhere = "PGPPassphraseDialog()";
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

    if ( passphrase )
    {
        SetPassphrase( passphrase );
        PGPFreeData ( ( void* ) passphrase );
        passphrase = NULL;
    }
    else
    {
        ret = FALSE;
    }

    return ret;
}

#endif

//  -  -   
void CSimplePGP::SetSignerIDstring ( string signerID )
{
    m_sSignerID = signerID;
}

PGPKeyRef CSimplePGP::GetFirstKeyFromSet( PGPKeySetRef &keyset )
{
    //  
    PGPError err = kPGPError_NoErr;
    // 
    PGPKeyRef keyref = kInvalidPGPKeyRef;
    //  
	PGPKeyListRef foundKeysList = kInvalidPGPKeyListRef;
    //    
	PGPKeyIterRef keyListIterator = kInvalidPGPKeyIterRef;
    // 
    PGPFilterRef filter = kInvalidPGPFilterRef;
    //  
    PGPKeySetRef foundUserKeys = kInvalidPGPKeySetRef;

    //    -   , 
    //  ,    ,
    //     
    if ( !m_sSignerID.empty() )
    {
        //      
  	    err = PGPNewUserIDStringFilter( m_context, m_sSignerID.c_str(), kPGPMatchSubString, &filter );

        if ( IsPGPError( err ) )
        {
            m_sWhere = "PGPNewUserIDStringFilter()  ";
            goto Exit;
        }

        //     
       	err = PGPFilterKeySet( keyset, filter, &foundUserKeys );

        if ( IsPGPError( err ) )
        {
            m_sWhere = "PGPFilterKeySet()  ";
            goto Exit;
        }

        //       
        err = PGPOrderKeySet( foundUserKeys, kPGPAnyOrdering, &foundKeysList );
    }
    else
    {
        //      ,  
        err = PGPOrderKeySet( keyset, kPGPAnyOrdering, &foundKeysList );
    }

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPOrderKeySet()  ";
        goto Exit;
    }

    //  
    err = PGPNewKeyIter( foundKeysList, &keyListIterator );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPNewKeyIter()  ";
        goto Exit;
    }

    //      
    err = PGPKeyIterNext( keyListIterator, &keyref );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPKeyIterNext()  ";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
        keyref = kInvalidPGPKeyRef;
    }

	if( PGPFilterRefIsValid( filter ) )
		PGPFreeFilter( filter );

    if( PGPKeyIterRefIsValid( keyListIterator ) )
		PGPFreeKeyIter( keyListIterator );

    if( PGPKeyListRefIsValid( foundKeysList ) )
		PGPFreeKeyList( foundKeysList );

  	if( PGPKeySetRefIsValid( foundUserKeys ) )
		PGPFreeKeySet( foundUserKeys );

    return keyref;
}

//  
//  ,     
BOOL CSimplePGP::SignFile2File( LPCTSTR inFileName,            //     
                                LPCTSTR outFileName,           //   
                                LPCTSTR keyFileName )          //     
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromFile( keyFileName, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputFile( m_context, inFileRef ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //  
                     PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                     //    
                     //   
                     m_optsSigning,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }
Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//  ,    
BOOL CSimplePGP::SignFile2File( LPCTSTR inFileName,            //   
                                LPCTSTR outFileName,     //   
                                LPCTSTR resourceName,          //   c  
                                LPCTSTR resourceType )         //  
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputFile( m_context, inFileRef ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //  
                     PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                     //    
                     //   
                     m_optsSigning,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//  -  ,  -  ,     
BOOL CSimplePGP::SignBuff2File( const VOID* inData,      //     
                                DWORD dwDataSize,        //  
                                LPCTSTR outFileName,     //   
                                LPCTSTR keyFileName )    //     
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromFile( keyFileName, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputBuffer( m_context,
                                      ( void* ) inData,
                                      ( PGPSize ) dwDataSize ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //  
                     PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                     //    
                     //   
                     m_optsSigning,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//  -  ,  -  ,    
BOOL CSimplePGP::SignBuff2File( const VOID* inData,      //     
                                DWORD dwDataSize,        //  
                                LPCTSTR outFileName,     //   
                                LPCTSTR resourceName,    //   c   
                                LPCTSTR resourceType )   //  
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef outFileRef = kInvalidPGPFileSpecRef;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, outFileName, &outFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", outFileName );
        goto Exit;
    }

    //  
    err = PGPEncode( m_context,
                     //  
                     PGPOInputBuffer( m_context,
                                      ( void* ) inData,
                                      ( PGPSize ) dwDataSize ),
                     //  
                     PGPOOutputFile( m_context, outFileRef ),
                     //  
                     PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                     //    
                     //   
                     m_optsSigning,
                     //    
                     PGPOLastOption( m_context ) );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    if ( PGPFileSpecRefIsValid( outFileRef ) )
        PGPFreeFileSpec( outFileRef );

    return ret;
}

//  -  ,  -  ,    .
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::SignFile2Buff( LPCTSTR inFileName,      //   
                                LPBYTE& OutData,         //     
                                DWORD& BuffSize,         //   
                                LPCTSTR keyFileName )    //     
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromFile( keyFileName, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        //  
        err = PGPEncode( m_context,
                        //  
                        PGPOInputFile( m_context, inFileRef ),
                        //  
                        PGPOOutputBuffer( m_context,
                                          ( void* ) OutData,
                                          ( PGPSize ) BuffSize,
                                          ( PGPSize * ) & BuffSize ),
                        //  
                        PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                        //    
                        //   
                        m_optsSigning,
                        //    
                        PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    return ret;
}

//  -  ,  -  ,    
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::SignFile2Buff( LPCTSTR inFileName,      //   
                                LPBYTE& OutData,         //     
                                DWORD& BuffSize,         //   
                                LPCTSTR resourceName,    //   c   
                                LPCTSTR resourceType )   //  
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //  
    PGPFileSpecRef inFileRef = kInvalidPGPFileSpecRef;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    err = PGPNewFileSpecFromFullPath( m_context, inFileName, &inFileRef );

    if ( IsPGPError( err ) )
    {
        m_sWhere = StrPrintf( "PGPNewFileSpecFromFullPath - '%s'  ", inFileName );
        goto Exit;
    }

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        //  
        err = PGPEncode( m_context,
                        //  
                        PGPOInputFile( m_context, inFileRef ),
                        //  
                        PGPOOutputBuffer( m_context,
                                          ( void* ) OutData,
                                          ( PGPSize ) BuffSize,
                                          ( PGPSize * ) & BuffSize ),
                        //  
                        PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                        //    
                        //   
                        m_optsSigning,
                        //    
                        PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
    }

Exit:

    if ( IsPGPError( err ) )
    {
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    if ( PGPFileSpecRefIsValid( inFileRef ) )
        PGPFreeFileSpec( inFileRef );

    return ret;
}

//  -  ,  -  ,    
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::SignBuff2Buff( const VOID* inData,      //     
                                DWORD dwDataSize,        //  
                                LPBYTE& OutData,         //     
                                DWORD& BuffSize,         //   
                                LPCTSTR keyFileName )    //     
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromFile( keyFileName, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        //  
        err = PGPEncode( m_context,
                         //  
                         PGPOInputBuffer( m_context,
                                          ( void* ) inData,
                                          ( PGPSize ) dwDataSize ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //  
                         PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                         //    
                         //   
                         m_optsSigning,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    return ret;
}

//  -  ,  -  ,    
//      
// OutData,     
//  ,  
//    
//   BuffSize
BOOL CSimplePGP::SignBuff2Buff( const VOID* inData,      //     
                                DWORD dwDataSize,        //  
                                LPBYTE& OutData,         //     
                                DWORD& BuffSize,         //   
                                LPCTSTR resourceName,    //   c  
                                LPCTSTR resourceType )   //  
{
    if ( !m_bIsInit )
        Init();

    BOOL ret = TRUE;

    //  
    PGPError err = kPGPError_NoErr;

    //    
    PGPKeySetRef secKeysSet = kInvalidPGPKeySetRef;

    //    
    if ( !ImportKeySetFromResorce( resourceName, resourceType, secKeysSet ) )
        return FALSE;

    //      
    PGPKeyRef secKeyRef = GetFirstKeyFromSet( secKeysSet );

    if ( secKeyRef == kInvalidPGPKeyRef )
    {
        ret = FALSE;
        goto Clear;
    }

    //   
    //  ,    PGPEncode 
    //  PGPOOutputBuffer    
    //   
    // ( 0      ,  1 )
    BuffSize = 1;

    //  
    do
    {
        delete[] OutData;
        //        
        //  
        OutData = new BYTE[ BuffSize ];

        //  
        err = PGPEncode( m_context,
                         //  
                         PGPOInputBuffer( m_context,
                                          ( void* ) inData,
                                          ( PGPSize ) dwDataSize ),
                         //  
                         PGPOOutputBuffer( m_context,
                                           ( void* ) OutData,
                                           ( PGPSize ) BuffSize,
                                           ( PGPSize * ) & BuffSize ),
                         //  
                         PGPOSignWithKey( m_context, secKeyRef, m_optsPassphrase, PGPOLastOption( m_context ) ),
                         //    
                         //   
                         m_optsSigning,
                         //    
                         PGPOLastOption( m_context ) );
    }
    while ( err == kPGPError_OutputBufferTooSmall );

    if ( IsPGPError( err ) )
    {
        m_sWhere = "PGPEncode()";
        ret = FALSE;
        //   
        PGPGetErrorString( err, sizeof( m_sWhat ), m_sWhat );
    }

Clear:
    // 
    if ( PGPKeySetRefIsValid( secKeysSet ) )
        PGPFreeKeySet( secKeysSet );

    return ret;
}
